/*******************************************************************************
 * File Name	: kb.h
 * Project	: Keycard
 * Date		: 2013/11/09
 * Version	: 1.0
 * Target MCU   : ATMEGA16U2
 * Tool Chain	: Atmel AVR Studio 4.19 730, avr-gcc, avr-libc
 * Author	: "Detlef Mueller" <detlef@gmail.com>
 * Release Notes:
 *
 * $Id$
 ******************************************************************************/

#ifndef	__kb_h__
#define	__kb_h__

//-------------------------------------------------------------------------------
// Keycard Hardware Specific Definitions

#define LED_PORT	PORTB
#define bLED_SYS	PB1

#define LED_on( b )	clr_bit( LED_PORT, (b) )
#define LED_off( b )	set_bit( LED_PORT, (b) )
#define LED_sts( b )	bit_is_clear( LED_PORT, (b) )

//-------------------------------------------------------------------------------
// kb.c interface

#define	SIG_RESET		TRUE	/* Signal reset to KB routines */
#define	SIG_MAINT		FALSE	/* Signal maintain to KB routines */

//-------------------------------------------------------------------------------

#define REP0_CHG		0x01	/* Report Changed flags */
#define REP1_CHG		0x02
#define REP2_CHG		0x04

//-------------------------------------------------------------------------------
// kbmain.c interface
//-------------------------------------------------------------------------------

#define sLED_on()		LED_on( bLED_SYS )
#define sLED_off()		LED_off( bLED_SYS )

#define susp_enter()			/* Don't need these w/o timer PWM */
#define susp_exit()

//-------------------------------------------------------------------------------

extern volatile uint8_t
    alarm ;				// Alarm signal, set every 500us

//-------------------------------------------------------------------------------

extern uint8_t
    wakeup( uint8_t reset ),		// Check for wakeup condition during suspend
    read_matrix( uint8_t reset ) ;	// Read and maintain key matrix

extern void
    led_pwm( void ),			// Maintain LED PWM
    sleep_led( uint8_t reset ),		// Maintain Sleep LED
    maint_leds( uint8_t reset ),	// Maintain LED's
    set_alarm( void ),			// Set up alarm
    hw_init( void ) ;			// Initialize HW

//-------------------------------------------------------------------------------

 #ifdef	__macro__

//-------------------------------------------------------------------------------

  #if __TECHKEYS

// http://www.techkeys.us

#define MACRO_1()								\
	{									\
	    /* uCC(U_AC_Home), uCC(U_None), uDms(3000), */			\
	    U_LftCtrl, U_L, U_None,						\
	    U_H, U_T, U_None, U_T, U_P, U_LftShift, U_SemiCol, U_Slash,	U_None,	\
	    U_Slash, U_W, U_None, U_W, U_None, U_W, U_Dot,			\
	    U_T, U_E, U_C, U_H, U_K, U_E, U_Y, U_S, U_Dot, U_U, U_S,		\
	    U_Enter, U_None							\
	}

// mailto:info@techkeys.us

#define MACRO_2()								\
	{									\
	    U_LftCtrl, U_L, U_None,						\
	    U_M, U_A, U_I, U_L, U_T, U_O, U_LftShift, U_SemiCol,		\
	    U_I, U_N, U_F, U_O, U_LftShift, U_2,				\
	    U_T, U_E, U_C, U_H, U_K, U_E, U_Y, U_S, U_Dot, U_U, U_S,		\
	    U_Enter, U_None							\
	}

// http://www.facebook.com/techkeysus

#define MACRO_3()								\
	{									\
	    U_LftCtrl, U_L, U_None,						\
	    U_H, U_T, U_None, U_T, U_P, U_LftShift, U_SemiCol, U_Slash, U_None,	\
	    U_Slash, U_W, U_None, U_W, U_None, U_W, U_Dot,			\
	    U_F, U_A, U_C, U_E, U_B, U_O, U_None, U_O, U_K, U_Dot, U_C, U_O,	\
	    U_M, U_Slash, U_T, U_E, U_C, U_H, U_K, U_E, U_Y, U_S, U_U, U_S,	\
	    U_Enter, U_None							\
	}

  #else

#define MACRO_1()								\
	{									\
	    U_I, U_F, U_Space,							\
	    U_LSft, U_9, U_Space, U_None, U_Space, U_LSft, U_0, U_Enter,	\
	    U_LSft, U_LBrc, U_Enter,						\
	    U_LSft, U_RBrc,							\
	    U_UArr, U_None, U_UArr, U_None,					\
	    U_RArr, U_None, U_RArr, U_None,					\
	    U_RArr, U_None, U_RArr, U_None					\
	}

#define MACRO_2()								\
	{									\
	    U_Enter, U_E, U_L, U_S, U_E, U_Enter,				\
	    U_LSft, U_LBrc, U_Enter,						\
	    U_LSft, U_RBrc,							\
	    U_UArr, U_Enter, U_None						\
	}

#define MACRO_3()								\
	{									\
	    U_F, U_O, U_R, U_Space,						\
	    U_LSft, U_9, U_Space, U_SemiCol, U_None, U_SCln, U_Space,		\
	    U_LSft, U_0, U_Enter, U_LSft, U_LBrc, U_Enter,			\
	    U_LSft, U_RBrc,							\
	    U_UArr, U_None, U_UArr, U_None,					\
	    U_RArr, U_None, U_RArr, U_None, U_RArr, U_None,			\
	    U_RArr, U_None, U_RArr, U_None, U_RArr, U_None			\
	}

  #endif

//-------------------------------------------------------------------------------

 #endif // __macro__

//-------------------------------------------------------------------------------

#endif	// __kb_h__
